/****************************************************************************
** $Id: rs_filterinterface.h,v 1.6 2003/11/23 22:25:34 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#ifndef RS_FILTERINTERFACE_H
#define RS_FILTERINTERFACE_H

#include "rs_graphic.h"

#include "rs_string.h"
#include "rs_stringlist.h"

#include "rs_debug.h"

/**
 * This is the interface that must be implemented for all 
 * format filter classes. The RS_Import and RS_Export classes 
 * use the methods defined in here to interact with the format
 * filter classes.
 *
 * @author Andrew Mustun
 */
class RS_FilterInterface {
public:
    /**
     * Constructor.
     *
     * Sets the entity container on which the format class inherited 
     * from this interface operates.
     */
    RS_FilterInterface() {
		//std::cout << "RS_FilterInterface\n";
        graphic = NULL;
    }

    /**
     * Destructor.
     */
    virtual ~RS_FilterInterface() {}

	/**
	 * @return Unique ID for this filter.
	 */
	virtual RS2::FormatType rtti() = 0;

    /**
     * Checks if this filter can import the given file type.
     *
     * @retval true if the filter can import the file type 
     * @retval false otherwise.
     */
    bool canImport(const RS_String& extension) {
        return !(importExtensions.find(extension.lower())==importExtensions.end());
    }

    /**
     * Checks if this filter can export the given file type.
     *
     * @return <code>true</code> if the filter can export the file type, 
     *         <code>false</code> otherwise.
     */
    bool canExport(const RS_String& extension) {
        return !(exportExtensions.find(extension.lower())==importExtensions.end());
    }

    /**
     * The implementation of this method in a inherited format
     * class should read a file from disk and put the entities
     * into the current entity container.
     */
    virtual void fileImport(RS_Graphic& graphic, const RS_String& file) = 0;

    /**
     * The implementation of this method in a inherited format
     * class should write the entities in the current entity container
     * to a file on the disk.
     */
    virtual void fileExport(RS_Graphic& graphic, const RS_String& file) = 0;

protected:
    /**
     * Adds a file extension which can be imported by this filter.
     */
    void addImportExtension(const RS_String& extension) {
        RS_DEBUG->print("Filter can import %s", extension.latin1());
        importExtensions += extension.lower();
    }

    /**
     * Adds a file extension which can be exported by this filter.
     */
    void addExportExtension(const RS_String& extension) {
        RS_DEBUG->print("Filter can export %s", extension.latin1());
        exportExtensions += extension.lower();
    }

protected:
    //! Pointer to the graphic we currently operate on.
    RS_Graphic* graphic;

    //! Vector of file extensions this filter can import.
    RS_StringList importExtensions;

    //! Vector of file extensions this filter can export.
    RS_StringList exportExtensions;
};

#endif
